// PICSLabel.java
// by Yang-hua Chu
// last update: 10/11/96

package w3c.pics.label;

import java.net.*;
import java.io.*;
import java.util.*;
import w3c.pics.util.*;

public class PICSLabel {
  public String version = "";
  public String serviceError = null;
  public String serviceErrorExplanation = "";
  public URL serviceID = null; 
  public PICSLabelOptions options = new PICSLabelOptions();
  private PICSSingleLabel singleLabels[] = {};  // a vector of PICSSingleLabel

  // index starts from 0
  public PICSSingleLabel getSingleLabel(int index)
    throws PICSLabelException
  {
    try { return singleLabels[index]; }
    catch (Exception e) {
      throw new PICSLabelException("Index out of bound: " + index);
    }
  }
  
  
  // add a PICS label to the label-list
  public int addSingleLabel(PICSSingleLabel inLabel)
  {
    PICSSingleLabel nlabels[] = new PICSSingleLabel[singleLabels.length + 1] ;
    System.arraycopy (singleLabels, 0, nlabels, 0, size()) ;
    singleLabels = nlabels;
    singleLabels[size()-1] = inLabel;
    return (size()-1);
  }
  
  public int size()
  {
    return singleLabels.length;
  }

  public PICSLabel(String inVersion)
  {
    version = inVersion;
  }
  
  //   parse the grammar:
  //   service-info :: serviceID service-error
  //   service-info :: serviceID option* labelword label*
  // caller must be sure that there is a label to be parsed
  public boolean parse(SExp exp) 
       throws PICSParseException
  {
    // service-info :: [serviceID] ...
    if (!exp.expect(exp.SEXP_STRING)) 
      throw new PICSParseException(exp.errorMsg("service-URL"));
    
    try {
      serviceID = new URL(exp.current().getString());
    } catch (MalformedURLException e) {
      throw new PICSParseException(exp.errorMsg("service-URL"));
    }
    
    // service-info :: serviceID [service-error]
    // service-error :: 'error' 'service-unavailable'
    if (exp.expect(exp.SEXP_IDENT, "error")) {
      
      // service-error :: 'error' 'service-unavailable'
      if (exp.expect(exp.SEXP_IDENT, "service-unavailable")) {
	serviceError = "service-unavailable";
	return true;
      }
      // service-error :: 'error' '(' 'request-denied' explanation* ')'
      if (exp.expect(exp.SEXP_SEXP)) {
	SExp nexp = exp.current();
	if (nexp.expect(nexp.SEXP_IDENT, "request-denied")) {
	  this.serviceError = "request-denied";
	  while (exp.expect(nexp.SEXP_STRING)) {
	    this.serviceErrorExplanation += " " + nexp.current().getString();
	  } //while
	  return true;
	} // if
      }
      throw new PICSParseException(exp.errorMsg("request-denied or service-unavailable"));
    } //if
    
    //service-info:: serviceID [option*] labelword label*
    options.parse(exp);

    //service-info:: serviceID option* [labelword] label*
    if (! (exp.expect(exp.SEXP_IDENT, "l") || 
	   exp.expect(exp.SEXP_IDENT, "labels"))) 
      throw new PICSParseException(exp.errorMsg("label word 'l'/'labels'"));
    
    
    // service-info:: serviceID option* labelword [label*]
    // label :: label-error | single-label | '(' single-label* ')'
    while (true) {
      PICSSingleLabel newSingleLabel = new PICSSingleLabel();
      if (!newSingleLabel.parse(exp))
	break;
      addSingleLabel(newSingleLabel);
    }
    return true;
  }


  public String toString()
  {
    String s = "\"" + serviceID.toString() + "\"\n";

    // error ( request-denied ["..."]  )
    // error ( service-unavailable )
    if (serviceError != null)
      if (serviceError.equals("request-denied"))
	if (serviceErrorExplanation.equals(""))
	  return s + " error ( request-denied )";
	else
	  return s+" error ( request-denied \""+serviceErrorExplanation+"\" )";
      else
	return s + " error ( service-unavailable )";
    
    s = s + options.toString();
    s += " labels";
    for (int i=0; i < size(); i++) {
      try { s += " " + getSingleLabel(i).toString(); }
      catch (PICSLabelException e) { continue; }
    }
    return s;
  }
}
  
