// LabelSource.java	last mod:	4/28/97	dshapiro


package w3c.pics.profiler;

import java.net.URL;
import java.net.MalformedURLException;

/**
 * Represents a source of PICS labels.  Contains all the information needed 
 * in order for the system to make a serviceinfo clause for this source.
 **/

public class LabelSource {

	URL name = null;
	URL bureauURL = null;	 
	URL ratURL = null;
	String shortname = null;
	String ratfile = null;
	boolean rurlmode = false;
	String bureauUnavailable = null;

	/**
	 * Creates a new LabelSource.
	 * @param ServiceURL The URL of the Ratings Service.
	 * @param sname The human readable name for this source.
	 * @param bureau The URL of the Label Bureau for this Service.
	 * @param rfile A String containing the text of the associated .rat file or 
	 * its URL.
	 * @param rmode A boolean indicating if <em>rfile</em> is a URL or the 
	 * actual .rat file.
	 **/

	public LabelSource(String ServiceURL, String sname, String bureau, 
		String rfile, boolean rmode) {
		try {
			name = new URL(ServiceURL);
			bureauURL = new URL(bureau);
		}
		catch (MalformedURLException e) {
			System.out.println("URL Error");
		}
		rurlmode = rmode;
		if (rurlmode) {
			try {
				ratURL = new URL(rfile);
			}
			catch (MalformedURLException e) {
				System.out.println("URL Error");
			}
		}
		ratfile = rfile;
		shortname = sname;
	}

	/**
	 * Creates a new, empty LabelSource.
	 **/

	public LabelSource() {

	}

	/**
	 * Returns the shortname for this LabelSource.
	 * @return The String value of the shortname clause.
	 **/

	public String getShortname() {
		return shortname;
	}

	/**
	 * Returns the name for this LabelSource.
	 * @return The String value of the name clause.
	 **/

	public String getName() {
		String servstring = name.toString();
		return servstring;
	}

	/**
	 * Returns the bureauURL for this LabelSource.
	 * @return The String value of the bureauURL clause.
	 **/

	public String getBureauURL() {
		String bstring = bureauURL.toString();
		return bstring;
	}

	/**
	 * Returns the text of the ratfile for this LabelSource.
	 * @return A String containing the text of the .rat file, replacing quotes 
	 * with hex.
	 **/

	public String getRatfile() {
		if (ratfile.equals(""))
			return null;
		else
			return(quoteFilter(ratfile));
	}

	/**
	 * Returns the text of the ratfile for this LabelSource.
	 * @return A String containing the text of the .rat file without replacing 
	 * quotes.
	 **/

	public String getUnparsedRatfile() {
		if (ratfile.equals(""))
			return null;
		else
			return(ratfile);
	}

	private String quoteFilter(String inputfile) {
		int qindx = 0;
		String output = inputfile;
		String front = inputfile;
		qindx = output.indexOf('"');
		while (qindx!=-1) {
			front = output.substring(0, qindx);
			front = front+"%22";
			front = front+output.substring(qindx+1);
			output = front;
			qindx = output.indexOf('"');
		}
		return output;
	}

	private String requoteFilter(String inputfile) {
		int qindx = 0;
		char[] input = inputfile.toCharArray();
		StringBuffer output = new StringBuffer(); 
		while (qindx<input.length) {
			if (input[qindx]!='%') {
				output.append(input[qindx]);
				qindx++;
				continue;
			}
			else {
				if (input[qindx+1]!='2') {
					output.append(input[qindx]);
					qindx++;
					continue;
				}
				else {
					if (input[qindx+2]!='2') {
						output.append(input[qindx]);
						qindx++;
						continue;
					}
					else {
						output.append('"');
						qindx = qindx+3;
						continue;
					}
				}
			}
		}
		return output.toString();
	}

	/**
	 * Returns the URL for the ratfile if it exists.
	 * @return A String containing the URL of the ratfile for this LabelSource.
	 **/

	public String getRatURL() {
		if (rurlmode) {
			if (ratURL==null)
				return "";
			else
				return(ratURL.toString());
		}
		else
			return "";
	}

	/**
	 * Returns the bureauUnavailable value for this LabelSource.
	 * @return The String value of the bureauUnavailable clause.
	 **/

	public String getBureauUnavail() {
		return bureauUnavailable;
	}

	/**
	 * Changes the name clause of this LabelSource.  If the URL supplied is 
	 * not valid, no change is made.
	 * @param n The new name URL as a String.
	 **/

	public void putName(String n) {
		URL newName = null;
		try {
			newName = new URL(n);
		}
		catch (MalformedURLException e) {
			return;
		}
		name = newName;
	}

	/**
	 * Changes the shortname clause of this LabelSource.
	 * @param sn The new shortname.
	 **/

	public void putShortname(String sn) {
		shortname = sn;
	}

	/**
	 * Changes the bureauURL clause of this LabelSource.  If the URL supplied 
	 * is not valid, no change is made.
	 * @param burl The new URL, as a String.
	 **/

	public void putBureauURL(String burl) {
		URL newB = null;
		try {
			newB = new URL(burl);
		}
		catch (MalformedURLException e) {
			return;
		}
		bureauURL = newB;
	}


	/**
	 * Changes the Ratfile URL for this LabelSource.  If the URL supplied is 
	 * not valid, no change is made.
	 * @param newurl The new URL as a String.
	 **/

	public void setRatURL(String newurl) {
		URL newR = null;
		try {
			newR = new URL(newurl);
		}
		catch (MalformedURLException e) {
			if (newurl.equals("")) {
				ratURL = null;
				rurlmode = false;
			}
			return;
		}
		ratURL = newR;
		rurlmode = true;
	}

	/**
	 * Changes the bureauUnavailable clause of this LabelSource.
	 * @param ba The new bureauUnavailable value.
	 **/

	public void putBureauUnavail(String ba) {
		bureauUnavailable = ba;
	}

	/**
	 * Changes the ratfile clause of this LabelSource.  It parses the string to 
	 * convert quotes back from hex.
	 * @param rfile The ratfile (either a file or a URL).
	 **/

	public void parseRatfile(String rfile) {
		ratfile = requoteFilter(rfile);
		// System.out.println("ratfile requoted: "+ratfile);
	}

	/**
	 * Returns a boolean indicating if the ratfile field of this LabelSource is 
	 * a URL of the text of an actual .rat file.
	 * @return A boolean, <strong>true</strong> if the ratfile field is a URL, 
	 * <strong>false</strong> if the ratfile field is a file.
	 **/

	public boolean getRmode() {
		return rurlmode;
	}


}